// R-tree Library. A simple 2d- C++ implementation of the R-tree structure
//
// Copyright (C) 2002-2003  Ondrej Pavlata.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// Contact information:
//  Mailing address:
//    Ondrej Pavlata
//    V. Nezvala 7a
//    466 02 Jablonec nad Nisou
//    Czech republic
//
//  Email:
//    o.j.p@worldonline.cz

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
#include <string.h>

#include <mgrtree/Assert.h>
#include <mgrtree/System.h>
#include <mgrtree/Rtree.h>
#include "SegRtSet.h"
#include "T_xx.h"

#undef EndOfIncludesSection

/*------------------------------------------------------------------------- o
  private definitions
o -------------------------------------------------------------------------*/

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
void
SegRtreeSet::removeGraySegs ()
{
  GraySegsBuf & buf = segList.graySegsBuf;
  int           i, n = buf.n;
  if ( n == 0 )
    return;
  for (i = 0; i < n; i++) {
    RecId  recId (buf.ids[i].entry);
    int  status =
    rtree.deleteRecord (recId);
    assert (status == SUCCESS);
  }
  printf ("%d records removed\n", n);
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
int
SegRtreeSet::createFromTextFile (
const char * fileName)
{
  int  status =
  segList.createFromTextFile (fileName);
  if ( SUCCESS != status )
    return  status;

  printf ("Building R-tree...\n");
  MgSystem::startTime();

  SegListPos pos (&segList);
  while ( SUCCESS == pos.nextStep() ) {
    RecId  recId (pos.memb); // build the recId from the memb-pointer
    status =
    rtree.insertRecord (recId);
    if ( SUCCESS != status )
      return  status;
  }
  printf ("%d records inserted in %.2f seconds\n", rtree.numRecords,
                                                   MgSystem::difTime());
  // deletion test
  removeGraySegs ();

  return  SUCCESS;
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
int
SegLocator::nextStep ()
{
  if ( SUCCESS != searcher.nextStep() )
    return  FAILURE;
  SegEntry * entry = (SegEntry *) searcher.cur.recId.rec;
  seg.set (entry->segment(), SegId (entry));
  return  SUCCESS;
}

/*------------------------------------------------------------------------- o
o ------------------------------------------------------------------------*/
void
SegRtreeSet::traverseRangeIncidences ()
{
  SegLocator locator_outer (this->rtree);
  SegLocator locator_inner (this->rtree);

  XyRange    outerRange;
  outerRange.setUnlimited ();

  locator_outer.setRange (outerRange);

  int  numCountedSegs = 0;

  MgSystem::startTime ();

  while ( SUCCESS == locator_outer.nextStep() ) {     // outer loop
    locator_inner.setRange (locator_outer.seg.range());
    locator_inner.setPreStart ();
    while ( SUCCESS == locator_inner.nextStep() ) {     // inner loop
      numCountedSegs++;
    }
  }
  // every segment should be counted twice
  assert (numCountedSegs % 2 == 0);

  printf ("\nRange incidences traversal result: %d incident pairs\n",
          numCountedSegs / 2);
  MgSystem::printDifTime ();
}

/*------------------------------------------------------------------------- o
o ------------------------------------------------------------------------*/
static void
test0 ()
{
  SegRtreeSet   segs;
  segs.createFromTextFile ("e:\\w\\1\\mgrtree\\segincid\\in\\SegList0.txt");
  segs.traverseRangeIncidences ();
}

/*------------------------------------------------------------------------- o
o ------------------------------------------------------------------------*/
void
testSegRtreeSet ()
{
  test0 ();
}


