// R-tree Library. A simple 2d- C++ implementation of the R-tree structure
//
// Copyright (C) 2002-2003  Ondrej Pavlata.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// Contact information:
//  Mailing address:
//    Ondrej Pavlata
//    V. Nezvala 7a
//    466 02 Jablonec nad Nisou
//    Czech republic
//
//  Email:
//    o.j.p@worldonline.cz

#ifndef  __segrtsetH__
#define  __segrtsetH__

#include <stdio.h>

#include <mgrtree/BasDef.h>
#include <mgrtree/Rtree.h>
#include "SegList.h"

#undef EndOfIncludesSection

// Segment's Record handle:
//   interprets given recId as a pointer to segment entry
class SegRh : public Rh {
 public:
  virtual XyRange range () {
    SegEntry * entry = (SegEntry *) recId.rec;
    return  entry->range();
  }
};

class SegRtree : public XyRtree {
 public:
  SegRtree () { setUfRep (); rhPrototype = new SegRh; }
 public:
};

// segment set equipped with an R-tree access
class SegRtreeSet {
 private:
  SegList    segList;  // segment proper container (owned)
 public:
  SegRtree   rtree;    // spatial access handle
 public:
  void init () {}
  SegRtreeSet () { init(); }
 public:
//  sbool isEmpty () { return segList.isEmpty(); }
  void removeGraySegs ();
  int createFromTextFile (const char * fileName);
  void traverseRangeIncidences ();
};

class SegLocator {
 private:
  Searcher   searcher;  // range searcher
 public:
  SegWid     seg;   // segment currently picked
 public:
  void init () {}
  SegLocator () { init (); };
  void init (SegRtree & rtree) { searcher.init (rtree); }
  SegLocator (SegRtree & rtree) { init (rtree); }
 public:
  void setRange (XyRange & r) { searcher.setRange (r); }
  void setPreStart () { searcher.setPreStart(); }
  int nextStep ();
};

inline XyRange
Abscissa::range () {
  AbEndPoint   llur[2], & ll = llur[0], & ur = llur[1];
  ll = start(); ur = end();
  // ll.x <= ur.x ensured, so only y-coords have to be considered
  if ( ll.y > ur.y ) MG_swap (int, ll.y, ur.y);
  return  XyRange (&ll.x);
}

inline XyRange
Segment::range () { return ab.range(); }

inline
SegId::SegId (RecId & recId) { entry = (SegEntry *) recId.rec; }

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/

#endif      /* __segrtsetH__ */






