#ifndef     __MGRTREE_csizestrH__
#define     __MGRTREE_csizestrH__

#include <mgrtree/BasDef.h>

#undef EndOfIncludesSection

class CharPtr {
  const char * ptr;
 public:
  CharPtr (const char * ptr) { this->ptr = ptr; }
  operator const char * () const { return ptr; }
};

class StringizedChar {
  char   twoChars[2];
 public:
  StringizedChar (char c) { twoChars[0] = c; twoChars[1] = '\0'; }
  operator const char * () const { return twoChars; }
};
inline StringizedChar stringizeChar (char c) {
  StringizedChar sc (c);  return  sc; }


// constant sized string
class CsizeStringBase  {
 protected:
  short   _len;
  short   charraySize;
 public:
  void init () { _len = 0; charraySize = 0;}
  CsizeStringBase () { init(); };
 protected:
  char * charray () const { return (char *) (this + 1); }
 public:
  const char * s () const { return charray(); }
  operator const char * () const { return s();}
//  operator char * () { return charray;}
  int len () const { return _len; } /* nonnegativ */
 public:
  void setCharray (const char * s, int len = -1) {
    if ( len == -1 )
      len = ::strlen (s);
    len = MG_min (len, charraySize - 1);
    ::strncpy (charray(), s, len);
    charray()[len] = '\0';
    _len = len;
  }
  void setStr (const CsizeStringBase & s) { setCharray (s.s(), s.len()); }
  void setChar (char c) { setCharray (stringizeChar(c), 1); }
 public:
  void appendCharray (const char * adCharray, short adLen = -1) {
    if ( adLen == -1 )
      adLen = ::strlen (adCharray);
    if ( adLen == 0 )
      return;
    adLen = MG_min (adLen, charraySize - 1 - _len);
    ::strncat (charray(), adCharray, adLen);
    _len += adLen;
  }
  void appendStr (const CsizeStringBase & s) {
    appendCharray (s.s(), s.len()); }
  void appendChar (char c) { appendCharray (stringizeChar(c), 1); }
 public:
 public:
  sbool isEqual (const CsizeStringBase & str, sbool isCaseSensitive) const {
    return isCaseSensitive ? ::strcmp (s(), str.s()) == 0 :
                             ::_stricmp (s(), str.s()) == 0;
  }
  sbool isEmpty () const { return _len == 0; }
 public:
  CsizeStringBase & operator= (const CsizeStringBase & s) {
    setStr (s); return *this; }
  CsizeStringBase & operator= (const char * s) {
    setCharray (s); return *this; }
  CsizeStringBase & operator= (char c) {
    setChar (c); return *this; }
  CsizeStringBase & operator+= (const CsizeStringBase & s) {
    appendStr (s); return *this; }
  CsizeStringBase & operator+= (const char * s) {
    appendCharray (s); return *this; }
  CsizeStringBase & operator+= (char c) {
    appendChar (c); return *this; }
};

template <short maxSize>
class CsizeString : public CsizeStringBase {
 protected:
  char  charrayBuf[maxSize]; // follows the base immediately
 public:
  void init () {
    CsizeStringBase::init(); charraySize = maxSize;
    MG_setZero (charrayBuf);
  }
  void swap () {
    int      i, n = len();
    for (i = 0; i < n / 2; i++)
      MG_swap (char, charrayBuf[i], charrayBuf[n-1-i]);
  }
  CsizeString () { init(); };
  CsizeString (const char * s) { init (); setCharray (s); }
  CsizeString (const CsizeStringBase & s) { init (); setStr (s); }
  CsizeString (char c) { init (); setChar (c); }
};

#endif      /* __MGRTREE_csizestrH__ */

