// R-tree Library. A simple 2d- C++ implementation of the R-tree structure
//
// Copyright (C) 2002-2003  Ondrej Pavlata.
//
// This library is free software; you can redistribute it and/or
// modify it under the terms of the GNU Lesser General Public
// License as published by the Free Software Foundation; either
// version 2.1 of the License, or (at your option) any later version.
//
// This library is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
// Lesser General Public License for more details.
//
// You should have received a copy of the GNU Lesser General Public
// License along with this library; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307 USA
//
// Contact information:
//  Mailing address:
//    Ondrej Pavlata
//    V. Nezvala 7a
//    466 02 Jablonec nad Nisou
//    Czech republic
//
//  Email:
//    o.j.p@worldonline.cz

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
#include <string.h>
#include <stdio.h>
#include <malloc.h>

#include "xxdef.h"

#include <mgrtree/BasDef.h>
#include <mgrtree/System.h>
#include "UfRep.h"

#include <mgrtree/Assert.h>

#undef EndOfIncludesSection
/*------------------------------------------------------------------------- o
       private definitions
o -------------------------------------------------------------------------*/

/*------------------------------------------------------------------------- o
       static global variables
o -------------------------------------------------------------------------*/

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
sbool
Member::hasFewSiblings ()
{
  assert (!isRoot());
  if ( isRootChild() )
    return  FALSE;
  return  n == rtree->minNodeMembs;
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
sbool
Member::hasRangeBoundaryTouch (
XyRangeI & rng)
{
  if ( isRoot() )
    return  FALSE;
  // assert (parent.rng.includes (rng));
  return  !parent.rng.includes_inner (rng);
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
sbool
Member::hasRangeBoundaryTouch ()
{
  return  hasRangeBoundaryTouch (this->rng);
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
void
Member::remove ()
{
  assert (!isRoot());
  sbool  isRngAdjustReq = hasRangeBoundaryTouch();

  entryValue().setEmpty();
  entry().setEmpty();
  patchEmptyWithEnd ();
  n--;
  node->nMembers = (char) n;

  while ( isRngAdjustReq ) {
    XyRangeI origParentRng = parent.rng;
    adjustParentRng ();
    upStep ();
    isRngAdjustReq = hasRangeBoundaryTouch (origParentRng);
  }
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
int
Deleter::reinsertSepLeafs ()
{
  if ( sep.isLeaf() )
    return  SUCCESS;

  LevelStepDriver   driver (sep.leafLevelIdx());
  Member            memb = sep; // assert (memb.isRoot());
  while ( SUCCESS == memb.nestStep (driver) ) {
    if ( memb.recId().isEqual (recId) )
      continue;  // don't reinsert the deleted record
    // use inserter call (instead of rtree->) to avoid numRecords counting
    Inserter (rtree).insertRecord (memb.id.recId());
  }
  return  SUCCESS;
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
int
Deleter::deleteRecord (
RecId & recId)
{
  this->recId = recId;

  sep.rtreeRootStep ();

  int  status =
  sep.findRecord (recId);
  if ( SUCCESS != status )
    return  FAILURE;

  while ( sep.hasFewSiblings() ) {
    sep.upStep ();
  }

  Member  memb = sep;
  sep.sepRootStep (&sepRoot);
  memb.remove();

  reinsertSepLeafs ();

  sep.destroyDescendantNodes ();

  return  SUCCESS;
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
sbool
UfXyRtreeRep::hasSingleRecord (
RecId & recId)
{
  if ( numRecords != 1 )
    return  FALSE;
  return  firstRootChild().recId().isEqual (recId);
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
void
UfXyRtreeRep::shorten ()
{
  assert (firstRootChild().n == 1 && numRecords != 1);
  Member  memb = firstRootChild();
  assert (memb.id.node != NULL);
  assert (root.rng.isEqual (memb.rng));
  memb.id.node->parent = NULL;
  memb.id.node->inParentIdx = -1;
  root.node = memb.id.node;
  free (memb.node);
  nLevels--;
}

/*------------------------------------------------------------------------- o
o -------------------------------------------------------------------------*/
int
UfXyRtreeRep::deleteRecord (
RecId & recId)
{
  if ( hasSingleRecord (recId) ) { // extra handling
    destroyAllNodes ();
    return  SUCCESS;
  }
  Deleter  deleter (this);
  deleter.deleteRecord (recId);
  numRecords--;
  if ( firstRootChild().n == 1 && numRecords != 1 ) {
    shorten ();
  }
  if ( owner != NULL ) {
    owner->overallRange.fromRngI (root.rng);
    owner->numRecords = numRecords;
  }
  return  SUCCESS; //???
}



